﻿/*
**
**                    MAP ZONE VIEWER - Version 1.0
**
** Map Zone Viewer loads an image file from hardrive, counts and displays
** the zones of the image divided by borderlines
**
** FreeImage library was used for the program to load images with different
** formats.
**
** Entirely designed and programmed by Caner'Trooper'Kurt on March 2021
** in Ankara, Turkey
**
** https://trooperthedreamer.com (all rights are reserved [(-:])
**
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <tchar.h>
#include <string>
#include <algorithm>
#include <array>
#include <stack>
#include "Map Zone Viewer.h"

//---------------------------------------------------------------------------
USEFORM("MainWindow.cpp", MapZoneViewer);
//---------------------------------------------------------------------------
int WINAPI _tWinMain(HINSTANCE, HINSTANCE, LPTSTR, int)
{
	try
	{
		Application->Initialize();
		Application->MainFormOnTaskBar = true;
		Application->CreateForm(__classid(TMapZoneViewer), &MapZoneViewer);
		Application->Run();
	}
	catch (Exception &exception)
	{
		Application->ShowException(&exception);
	}
	catch (...)
	{
		try
		{
			throw Exception("");
		}
		catch (Exception &exception)
		{
			Application->ShowException(&exception);
		}
	}
	return 0;
}
//---------------------------------------------------------------------------

//
// Classes
//

//---------------------------------------------------------------------------
// Exception Class
//---------------------------------------------------------------------------
MZV::Error::Exception::Exception(std::wstring message, std::wstring title)
 : mErrorMessage(message), mErrorTitle(title)
{}

std::wstring MZV::Error::Exception::getErrorMessage(void) const
{
	return mErrorMessage;
}

std::wstring MZV::Error::Exception::getErrorTitle(void) const
{
	return mErrorTitle;
}


#include <algorithm>

//---------------------------------------------------------------------------
// Vector 2D Class
//---------------------------------------------------------------------------
MZV::Math::Vector2D::Vector2D() : mX(0.0f), mY(0.0f)
{}

MZV::Math::Vector2D::Vector2D(float x, float y) : mX(x), mY(y)
{}

MZV::Math::Vector2D::Vector2D(const Vector2D& vector) : mX(vector.mX), mY(vector.mY)
{}


MZV::Math::Vector2D& MZV::Math::Vector2D::operator=(const Vector2D& vector)
{
	mX = vector.mX;
	mY = vector.mY;

	return *this;
}

float MZV::Math::Vector2D::operator[](unsigned int element) const
{
	switch (element) {
		case 0 :
			return mX;
		break;

		case 1 :
			return mY;
		break;

		default:
			return 0.0f;
		break;
	}
}

float& MZV::Math::Vector2D::operator[](unsigned int element)
{
	switch (element) {
		case 0:
			return mX;
		break;

		case 1:
			return mY;
		break;

		default:
			return mX;
		break;
	}
}

bool MZV::Math::Vector2D::operator==(const Vector2D& vector) const
{
	return AreEqual(mX, vector.mX) && AreEqual(mY, vector.mY);
}

bool MZV::Math::Vector2D::operator!=(const Vector2D& vector) const
{
	return !(*this == vector);
}

MZV::Math::Vector2D MZV::Math::Vector2D::operator*(float scalar) const
{
	return Vector2D(mX * scalar, mY * scalar);
}

MZV::Math::Vector2D MZV::Math::Vector2D::operator/(float scalar) const
{
	return Vector2D(mX / scalar, mY / scalar);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::operator*=(float scalar)
{
	mX *= scalar;
	mY *= scalar;

	return *this;
}

MZV::Math::Vector2D& MZV::Math::Vector2D::operator/=(float scalar)
{
	mX /= scalar;
	mY /= scalar;

	return *this;
}


MZV::Math::Vector2D MZV::Math::Vector2D::operator*(const Matrix3x3& matrix) const
{
	return Vector2D(mX * matrix.m11 + mY * matrix.m12 + matrix.m13, mX * matrix.m21 + mY * matrix.m22 + matrix.m23);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::operator*=(const Matrix3x3& matrix)
{
	return *this = *this * matrix;
}

MZV::Math::Vector2D MZV::Math::Vector2D::operator+(const Vector2D& vector) const
{
	return Vector2D(mX + vector.mX, mY + vector.mY);
}

MZV::Math::Vector2D MZV::Math::Vector2D::operator-(const Vector2D& vector) const
{
	return Vector2D(mX - vector.mX, mY - vector.mY);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::operator+=(const Vector2D& vector)
{
	mX += vector.mX;
	mY += vector.mY;

	return *this;
}

MZV::Math::Vector2D& MZV::Math::Vector2D::operator-=(const Vector2D& vector)
{
	mX -= vector.mX;
	mY -= vector.mY;

	return *this;
}

float MZV::Math::Vector2D::getX(void) const
{
	return mX;
}

void MZV::Math::Vector2D::setX(float x)
{
	mX = x;
}

float MZV::Math::Vector2D::getY(void) const
{
	return mY;
}

void MZV::Math::Vector2D::setY(float y)
{
	mY = y;
}

bool MZV::Math::Vector2D::isZero(void) const
{
	return IsZero(mX) && IsZero(mY);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::setToZero(void)
{
	mX = mY = 0.0f;

	return *this;
}

MZV::Math::Vector2D MZV::Math::Vector2D::getNegate(void) const
{
	return Vector2D(mX * -1, mY * -1);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::setToNegate(void)
{
	mX *= -1;
	mY *= -1;

	return *this;
}

float MZV::Math::Vector2D::getMagnitude(void) const
{
	return SquareRoot(mX * mX + mY * mY);
}

MZV::Math::Vector2D& MZV::Math::Vector2D::normalize(void)
{
	float magnitude = getMagnitude();

	mX /= magnitude;
	mY /= magnitude;

	return *this;
}

MZV::Math::Vector2D& MZV::Math::Vector2D::round(void)
{
	mX = Round(mX);
	mY = Round(mY);

	return *this;
}

MZV::Math::Vector2D& MZV::Math::Vector2D::clean(void)
{
	if (IsZero(mX)) { mX = 0.0f; }
	if (IsZero(mY)) { mY = 0.0f; }

	return *this;
}

//---------------------------------------------------------------------------
// Matrix 3x3 Class
//---------------------------------------------------------------------------
MZV::Math::Matrix3x3::Matrix3x3()
 : m11(0.0f), m12(0.0f), m13(0.0f),
   m21(0.0f), m22(0.0f), m23(0.0f),
   m31(0.0f), m32(0.0f), m33(0.0f)
{}

MZV::Math::Matrix3x3::Matrix3x3(float first, float second, float third,
								float fourth, float fifth, float sixth,
								float seventh, float eighth, float ninth)
 : m11(first)  , m12(second), m13(third),
   m21(fourth) , m22(fifth) , m23(sixth),
   m31(seventh), m32(eighth), m33(ninth)

{}

MZV::Math::Matrix3x3::Matrix3x3(const Matrix3x3& matrix)
 : m11(matrix.m11), m12(matrix.m12), m13(matrix.m13),
   m21(matrix.m21), m22(matrix.m22), m23(matrix.m23),
   m31(matrix.m31), m32(matrix.m32), m33(matrix.m33)
{}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::operator=(const Matrix3x3& matrix)
{
	m11 = matrix.m11;
	m12 = matrix.m12;
	m13 = matrix.m13;
	m21 = matrix.m21;
	m22 = matrix.m22;
	m23 = matrix.m23;
	m31 = matrix.m31;
	m32 = matrix.m32;
	m33 = matrix.m33;

	return *this;
}

float MZV::Math::Matrix3x3::operator()(unsigned int row, unsigned int col) const
{
	switch (row) {
		case 1 :
			switch (col) {
				case 1 :
					return m11;
				break;

				case 2 :
					return m12;
				break;

				case 3 :
					return m13;
				break;

				default:
					return 0.0f;
				break;
			}
		break;

		case 2:
			switch (col) {
				case 1:
					return m21;
				break;

				case 2:
					return m22;
				break;

				case 3:
					return m23;
				break;

				default:
					return 0.0f;
				break;
			}
		break;

		case 3:
			switch (col) {
				case 1:
					return m31;
				break;

				case 2:
					return m32;
				break;

				case 3:
					return m33;
				break;

				default:
					return 0.0f;
				break;
			}
		break;

		default:
			return 0.0f;
		break;
	}
}

float& MZV::Math::Matrix3x3::operator()(unsigned int row, unsigned int col)
{
	switch (row) {
		case 1:
			switch (col) {
				case 1:
					return m11;
				break;

				case 2:
					return m12;
				break;

				case 3:
					return m13;
				break;

				default:
					return m33;
				break;
			}
			break;

		case 2:
			switch (col) {
				case 1:
					return m21;
				break;

				case 2:
					return m22;
				break;

				case 3:
					return m23;
				break;

				default:
					return m33;
				break;
			}
			break;

		case 3:
			switch (col) {
				case 1:
					return m31;
				break;

				case 2:
					return m32;
				break;

				case 3:
					return m33;
				break;

				default:
					return m33;
				break;
			}
			break;

		default:
			return m33;
		break;
	}
}

MZV::Math::Matrix3x3 MZV::Math::Matrix3x3::operator*(float scalar) const
{
	return Matrix3x3(m11 * scalar, m12 * scalar, m13 * scalar,
					 m21 * scalar, m22 * scalar, m23 * scalar,
				     m31 * scalar, m32 * scalar, m33 * scalar );
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::operator*=(float scalar)
{
	m11 *= scalar;
	m12 *= scalar;
	m13 *= scalar;
	m21 *= scalar;
	m22 *= scalar;
	m23 *= scalar;
	m31 *= scalar;
	m32 *= scalar;
	m33 *= scalar;

	return *this;
}

MZV::Math::Matrix3x3 MZV::Math::Matrix3x3::operator/(float scalar) const
{
	return Matrix3x3(m11 / scalar, m12 / scalar, m13 / scalar,
		             m21 / scalar, m22 / scalar, m23 / scalar,
		             m31 / scalar, m32 / scalar, m33 / scalar);
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::operator/=(float scalar)
{
	m11 /= scalar;
	m12 /= scalar;
	m13 /= scalar;
	m21 /= scalar;
	m22 /= scalar;
	m23 /= scalar;
	m31 /= scalar;
	m32 /= scalar;
	m33 /= scalar;

	return *this;
}



MZV::Math::Matrix3x3 MZV::Math::Matrix3x3::operator*(const Matrix3x3& matrix) const
{
	return Matrix3x3(m11 * matrix.m11 + m21 * matrix.m12 + m31 * matrix.m13,
				     m12 * matrix.m11 + m22 * matrix.m12 + m32 * matrix.m13,
		             m13 * matrix.m11 + m23 * matrix.m12 + m33 * matrix.m13,
		             m11 * matrix.m21 + m21 * matrix.m22 + m31 * matrix.m23,
		             m12 * matrix.m21 + m22 * matrix.m22 + m32 * matrix.m23,
		             m13 * matrix.m21 + m23 * matrix.m22 + m33 * matrix.m23,
		             m11 * matrix.m31 + m21 * matrix.m32 + m31 * matrix.m33,
		             m12 * matrix.m31 + m22 * matrix.m32 + m32 * matrix.m33,
		             m13 * matrix.m31 + m23 * matrix.m32 + m33 * matrix.m33);
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::operator*=(const Matrix3x3& matrix)
{
	return *this = *this * matrix;
}

MZV::Math::Vector2D MZV::Math::Matrix3x3::operator*(const Vector2D& vector) const
{
	return Vector2D(vector.mX * m11 + vector.mY * m12 + m31, vector.mX * m21 + vector.mY * m22 + m32);
}

MZV::Math::Matrix3x3 MZV::Math::Matrix3x3::getTranspose(void) const
{
	return Matrix3x3(m11, m21, m31,
		             m12, m22, m32,
		             m13, m23, m33);
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToTranspose(void)
{
	std::swap(m12, m21);
	std::swap(m23, m32);
	std::swap(m13, m31);

	return *this;
}

bool MZV::Math::Matrix3x3::isIdentity(void) const
{
	return AreEqual(m11, 1.0f) && IsZero(m12)         && IsZero(m13) &&
		   IsZero(m21)         && AreEqual(m22, 1.0f) && IsZero(m23) &&
	  	   IsZero(m31)         && IsZero(m32)         && AreEqual(m33, 1.0f);
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToIdentity(void)
{
	m11 = m22 = m33 = 1.0f;
	m12 = m13 = m21 = m23 = m31 = m32 = 0.0f;

	return *this;
}

bool MZV::Math::Matrix3x3::isZero(void) const
{
	return IsZero(m11) && IsZero(m12) && IsZero(m13) &&
		   IsZero(m21) && IsZero(m22) && IsZero(m23) &&
		   IsZero(m31) && IsZero(m32) && IsZero(m33);
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToZero(void)
{
	m11 = m12 = m13 = m21 = m22 = m23 = m31 = m32 = m33 = 0.0f;

	return *this;
}

float MZV::Math::Matrix3x3::getDeterminant(void) const
{
	return (m11 * ((m22 * m33) - (m23 * m32))) - (m12 * ((m21 * m33) - (m31 * m23))) + (m13 * ((m21 * m32) - (m31 * m22)));
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToTranslation(float translateX, float translateY)
{
	m11 = 1.0f;       m12 = 0.0f;       m13 = 0.0f;
	m21 = 0.0f;       m22 = 1.0f;       m23 = 0.0f;
	m31 = translateX; m32 = translateY; m33 = 1.0f;

	return *this;
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToRotation(float angle)
{
	m11 = Cosine(angle); m12 = -Sine(angle);  m13 = 1.0f;
	m21 = Sine(angle);   m22 = Cosine(angle); m23 = 0.0f;
	m31 = 0.0f;          m32 = 0.0f;          m33 = 1.0f;

	return *this;
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::setToScale(float scaleX, float scaleY)
{
	m11 = scaleX; m12 = 0.0f;   m13 = 0.0f;
	m21 = 0.0f;   m22 = scaleY; m23 = 0.0f;
	m31 = 0.0f;   m32 = 0.0f;   m33 = 1.0f;

	return *this;
}

MZV::Math::Matrix3x3& MZV::Math::Matrix3x3::clean(void)
{
	if (IsZero(m11)) { m11 = 0.0f; }
	if (IsZero(m12)) { m12 = 0.0f; }
	if (IsZero(m13)) { m13 = 0.0f; }
	if (IsZero(m21)) { m21 = 0.0f; }
	if (IsZero(m22)) { m22 = 0.0f; }
	if (IsZero(m23)) { m23 = 0.0f; }
	if (IsZero(m31)) { m31 = 0.0f; }
	if (IsZero(m32)) { m32 = 0.0f; }
	if (IsZero(m33)) { m33 = 0.0f; }

	return *this;
}

int MZV::Math::Absolute(int number)
{
	return number > 0 ? number : (number * -1);
}

float MZV::Math::Absolute(float number)
{
	return fabsf(number);
}

int MZV::Math::Min(int number1, int number2)
{
	if(number1 <= number2) {
		return number1;
	}

	return number2;
}

float MZV::Math::Min(float number1, float number2)
{
    if(number1 <= number2) {
		return number1;
	}

	return number2;
}

int MZV::Math::Max(int number1, int number2)
{
	if(number1 >= number2) {
		return number1;
	}

	return number2;
}

float MZV::Math::Max(float number1, float number2)
{
    if(number1 >= number2) {
		return number1;
	}

	return number2;
}

bool MZV::Math::IsOdd(int number)
{
	return number % 2;
}

bool MZV::Math::IsEven(int number)
{
	return !(number % 2);
}

bool MZV::Math::IsZero(float number)
{
	return Absolute(number) <= EPSILON;
}

bool MZV::Math::AreEqual(float number1, float number2)
{
	return Absolute(number1 - number2) <= EPSILON;
}

float MZV::Math::Round(float number)
{
	return std::roundf(number);
}

float MZV::Math::Ceil(float number)
{
	return std::ceilf(number);
}

float MZV::Math::Floor(float number)
{
	return std::floorf(number);
}

float MZV::Math::Square(float number)
{
	return number * number;
}

int MZV::Math::Square(int number)
{
	return number * number;
}

float MZV::Math::Power(float base, float exponent)
{
	return powf(base, exponent);
}

float MZV::Math::SquareRoot(float number)
{
	return sqrtf(number);
}

float MZV::Math::DegreeToRadian(float degree)
{
	return PI * degree / 180.0f;
}

float MZV::Math::RadianToDegree(float radian)
{
	return 180.0f / PI * radian;
}

float MZV::Math::Sine(float degree)
{
	return std::sinf(DegreeToRadian(degree));
}

float MZV::Math::Cosine(float degree)
{
	return std::cosf(DegreeToRadian(degree));
}

float MZV::Math::Tangent(float degree)
{
	return std::tanf(DegreeToRadian(degree));
}

float MZV::Math::ArcSine(float degree)
{
	return std::asinf(DegreeToRadian(degree));
}

float MZV::Math::ArcCosine(float degree)
{
	return std::cosf(DegreeToRadian(degree));
}

float MZV::Math::ArcTangent(float degree)
{
	return std::atanf(DegreeToRadian(degree));
}

float MZV::Math::Distance(Vector2D& vector1, Vector2D& vector2)
{
	return SquareRoot(Square(vector2.getX() - vector1.getX()) + Square(vector2.getY() - vector1.getY()));
}

float MZV::Math::AngleBetween(Vector2D& vector1, Vector2D& vector2)
{
	float mag1 = vector1.getMagnitude();
	float mag2 = vector2.getMagnitude();

	if (IsZero(mag1) || IsZero(mag2)) {
		return 0.0f;
	}

	return ArcCosine(DotProduct(vector1, vector2) / mag1 * mag2);
}

float MZV::Math::DotProduct(Vector2D& vector1, Vector2D& vector2)
{
	return vector1.getX() * vector2.getX() + vector1.getY() * vector2.getY();
}

float MZV::Math::CrossProduct(Vector2D& vector1, Vector2D& vector2)
{
	return vector1.getX() * vector2.getY() - vector1.getY() * vector2.getX();
}

bool MZV::Math::GetLineSegmentIntersection(POINT p0, POINT p1, POINT p2, POINT p3, Vector2D& intersection)
{
	int A1          = p1.y - p0.y;
	int B1          = p0.x - p1.x;
	int C1          = A1 * p0.x + B1 * p0.y;
	int A2          = p3.y - p2.y;
	int B2          = p2.x - p3.x;
	int C2          = A2 * p2.x + B2 * p2.y;
    int denominator = A1 * B2 - A2 * B1;

	if (denominator == 0) {
		return false;
	}

	float intersectX = (float)(B2 * C1 - B1 * C2) / denominator;
	float intersectY = (float)(A1 * C2 - A2 * C1) / denominator;
	float rx0        = (intersectX - p0.x) / (p1.x - p0.x);
	float ry0        = (intersectY - p0.y) / (p1.y - p0.y);
	float rx1		 = (intersectX - p2.x) / (p3.x - p2.x);
	float ry1		 = (intersectY - p2.y) / (p3.y - p2.y);

	if (((rx0 >= 0.0f && rx0 <= 1.0f) || (ry0 >= 0.0f && ry0 <= 1.0f)) &&
		((rx1 >= 0.0f && rx1 <= 1.0f) || (ry1 >= 0.0f && ry1 <= 1.0f))) {
		intersection.setX(intersectX);
		intersection.setY(intersectY);
		return true;
	} else {
		return false;
	}
}

MZV::Math::Vector2D operator*(float scalar, MZV::Math::Vector2D& vector)
{
	return MZV::Math::Vector2D(scalar * vector.getX(), scalar * vector.getY());
}

MZV::Math::Matrix3x3 operator*(float scalar, MZV::Math::Matrix3x3& matrix)
{
	return MZV::Math::Matrix3x3(matrix(1,1) * scalar, matrix(1,2) * scalar, matrix(1,3) * scalar,
								matrix(2,1) * scalar, matrix(2,2) * scalar, matrix(2,3) * scalar,
		                        matrix(3,1) * scalar, matrix(3,2) * scalar, matrix(3,3) * scalar);
}



//---------------------------------------------------------------------------
// Graphics Element Class
//---------------------------------------------------------------------------
MZV::Graphics::GraphicsElement::GraphicsElement()
 : mName(), mWidth(0), mHeight(0), mBits(nullptr)
{}

MZV::Graphics::GraphicsElement::GraphicsElement(const std::wstring& name, const BYTE* bits, const int width, const int height)
 : mName(name), mWidth(width), mHeight(height), mBits(nullptr)
{
	if (bits) {
		if (!setBits(bits, width, height)) {
			reset();
		}
	}
}

MZV::Graphics::GraphicsElement::GraphicsElement(const  MZV::Graphics::GraphicsElement& element)
 : mName(element.mName), mWidth(element.mWidth), mHeight(element.mHeight), mBits(nullptr)
{
	if (!setBits(element.mBits, element.mWidth, element.mHeight)) {
		reset();
	}
}

MZV::Graphics::GraphicsElement::~GraphicsElement()
{
	reset();
}

const MZV::Graphics::GraphicsElement& MZV::Graphics::GraphicsElement::operator=(const  MZV::Graphics::GraphicsElement& element)
{
	mName   = element.mName;
	mWidth  = element.mWidth;
	mHeight = element.mHeight;
	mBits   = nullptr;

	if (!setBits(element.mBits, element.mWidth, element.mHeight)) {
		reset();
	}

	return *this;
}

bool MZV::Graphics::GraphicsElement::operator==(const GraphicsElement& element) const
{
	if (element.mWidth == mWidth && element.mHeight == mHeight) {
		for (int i = 0; i < mWidth * mHeight * 4; ++i) {
			if (element.mBits[i] != mBits[i]) {
				return false;
			}
		}
	} else {
		return false;
	}

	return true;
}

bool MZV::Graphics::GraphicsElement::operator!=(const GraphicsElement& element) const
{
	return !(*this == element);
}

const std::wstring& MZV::Graphics::GraphicsElement::getName(void) const
{
	return mName;
}

void MZV::Graphics::GraphicsElement::setName(const std::wstring& name)
{
	mName = name;
}

int MZV::Graphics::GraphicsElement::getWidth(void) const
{
	return mWidth;
}

bool MZV::Graphics::GraphicsElement::setWidth(const int width, const Color fillColor, bool scale)
{
	if (mBits == nullptr) {
		mWidth = width;

		return true;
	}

	BYTE* newBits = MZV::Graphics::SetBitsWidth(mBits, mWidth, mHeight, width, mHeight, fillColor, scale);

	if (newBits == nullptr) {
		return false;
	} else {
		if (setBits(newBits, width, mHeight)) {
			delete[] newBits;

			mWidth  = width;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	}
}

int MZV::Graphics::GraphicsElement::getHeight(void) const
{
	return mHeight;
}

bool MZV::Graphics::GraphicsElement::setHeight(const int height, const Color fillColor, bool scale)
{
	if (mBits == nullptr) {
		mHeight = height;

		return true;
	}

	BYTE* newBits = SetBitsHeight(mBits, mWidth, mHeight, mWidth, height, fillColor, scale);

	if (newBits == nullptr) {
		return false;
	} else {
		if (setBits(newBits, mWidth, height)) {
			delete[] newBits;

			mHeight = height;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	}
}

BYTE* MZV::Graphics::GraphicsElement::getBits(void) const
{
	return mBits;
}

bool MZV::Graphics::GraphicsElement::setBits(const BYTE* bits, const int width, const int height)
{
	 mBits = SetBits(bits, width, height);

	if (mBits == nullptr) {
		reset();

		return false;
	} else {
		mWidth  = width;
		mHeight = height;

		return true;
	}
}

bool MZV::Graphics::GraphicsElement::getPixelColor(const int x, const int y, MZV::Graphics::Color& color) const
{
	try {
		if(x < 0 || y < 0 || x >= mWidth || y >= mHeight) {
			throw Error::Exception(L"Grafik elenmanı koordinat bilgisi hatalı!",L"Grafik Elemanı Erişim Hatası");
		}

		color.mBlue  = mBits[(y * mWidth * 4) + (x * 4 + 0)];
		color.mGreen = mBits[(y * mWidth * 4) + (x * 4 + 1)];
		color.mRed   = mBits[(y * mWidth * 4) + (x * 4 + 2)];
		color.mAlpha = mBits[(y * mWidth * 4) + (x * 4 + 3)];

		return true;
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(),L"Grafik Elemanı Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Grafik Elemanı Erişim Hatası");

		return false;
	}
}

bool MZV::Graphics::GraphicsElement::setPixelColor(const int x, const int y, Color color)
{
	try {
		if(x < 0 || y < 0 || x >= mWidth || y >= mHeight) {
			throw Error::Exception(L"Grafik elemanı koordinat bilgisi hatalı!",L"Grafik Elemanı Düzenleme Hatası");
		}

		mBits[(y * mWidth * 4) + (x * 4 + 0)] = color.mBlue;
		mBits[(y * mWidth * 4) + (x * 4 + 1)] = color.mGreen;
		mBits[(y * mWidth * 4) + (x * 4 + 2)] = color.mRed;
		mBits[(y * mWidth * 4) + (x * 4 + 3)] = color.mAlpha;

		return true;
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(),L"Grafik Elemanı Düzenleme Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Grafik Elemanı Düzenleme Hatası");

		return false;
	}
}

bool MZV::Graphics::GraphicsElement::rotate(float angle)
{
	int newWidth{ 0 }, newHeight{ 0 };

	BYTE* newBits = RotateBits(mBits, mWidth, mHeight, angle, &newWidth, &newHeight);

	if (newBits) {
		if (setBits(newBits, newWidth, newHeight)) {
			delete[] newBits;

			mWidth  = newWidth;
			mHeight = newHeight;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	} else {
		return false;
	}
}

bool MZV::Graphics::GraphicsElement::scale(int width, int height)
{
	BYTE* newBits = ScaleBits(mBits, mWidth, mHeight, width, height);

	if (newBits) {
		if (setBits(newBits, width, height)) {
			delete[] newBits;

			mWidth  = width;
			mHeight = height;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	} else {
		return false;
	}
}

bool MZV::Graphics::GraphicsElement::scale(RECT r)
{
	return scale((int)(r.right - r.left), (int)(r.bottom - r.top));
}

bool MZV::Graphics::GraphicsElement::flipVertical(void)
{
	BYTE* newBits = FlipBitsVertical(mBits, mWidth, mHeight);

	if (newBits) {
		if (setBits(newBits, mWidth, mHeight)) {
			delete[] newBits;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	} else {
		return false;
	}
}

bool MZV::Graphics::GraphicsElement::flipHorizontal(void)
{
	BYTE* newBits = FlipBitsHorizontal(mBits, mWidth, mHeight);

	if (newBits) {
		if (setBits(newBits, mWidth, mHeight)) {
			delete[] newBits;

			return true;
		} else {
			delete[] newBits;

			reset();

			return false;
		}
	} else {
		return false;
	}
}

void MZV::Graphics::GraphicsElement::reset(void)
{
	mWidth  = 0;
	mHeight = 0;

	if (mBits) {
		delete[] mBits;
		mBits = nullptr;
	}
}

//---------------------------------------------------------------------------
// Image Class
//---------------------------------------------------------------------------

MZV::Graphics::Image::Image()
: GraphicsElement(),
  mFileName(std::wstring{}), mFormat(FIF_UNKNOWN), mBPP(0), mPalette(nullptr), mDIB(nullptr)
{}

MZV::Graphics::Image::Image(const std::wstring& fileName, const std::wstring& name)
 : GraphicsElement(name),
   mFileName(fileName), mFormat(FIF_UNKNOWN), mBPP(0), mPalette(nullptr), mDIB(nullptr)
{
	load();
}

MZV::Graphics::Image::Image(const std::wstring& name, const BYTE* bits, const int width, const int height)
: GraphicsElement(name, bits, width, height),
  mFileName(std::wstring{}), mFormat(FIF_UNKNOWN), mBPP(0), mPalette(nullptr)
{}

MZV::Graphics::Image::Image(const Image& image)
: GraphicsElement(image.mName, image.mBits, image.mWidth, image.mHeight),
  mFileName(image.mFileName), mFormat(image.mFormat), mBPP(image.mBPP), mPalette(nullptr), mDIB(nullptr)
{
	if (image.mPalette) {
		setPalette(image.mPalette);
	}

	if (image.mDIB) {
		setDIB(image.mDIB);
	}
}

MZV::Graphics::Image::Image(const Image& image, const std::wstring& name)
 : GraphicsElement(name, image.mBits, image.mWidth, image.mHeight),
   mFileName(image.mFileName), mFormat(image.mFormat), mBPP(image.mBPP), mPalette(nullptr), mDIB(nullptr)
{
	if (image.mPalette) {
		setPalette(image.mPalette);
	}

	if (image.mDIB) {
		setDIB(image.mDIB);
	}
}

MZV::Graphics::Image::~Image()
{
	reset();
}

const MZV::Graphics::Image& MZV::Graphics::Image::operator=(const Image& image)
{
	// check if rasters have the same data
	if (*this == image) {
		return *this;
	}

	mName     = image.mName;
	mFileName = image.mFileName;
	mFormat   = image.mFormat;
	mWidth    = image.mWidth;
	mHeight   = image.mHeight;

	if (image.mPalette) {
		setPalette(image.mPalette);
	}

	if (image.mDIB) {
		setDIB(image.mDIB);
	}

	if (!setBits(image.mBits, mWidth, mHeight)) {
		reset();
	}

	return *this;
}

const std::wstring& MZV::Graphics::Image::getFileName(void) const
{
	return mFileName;
}

void MZV::Graphics::Image::setFileName(const std::wstring& fileName)
{
	mFileName = fileName;
}

const FREE_IMAGE_FORMAT MZV::Graphics::Image::getFormat(void) const
{
	return mFormat;
}

void MZV::Graphics::Image::setFormat(const FREE_IMAGE_FORMAT format)
{
	mFormat = format;
}

int MZV::Graphics::Image::getBPP(void) const
{
	return mBPP;
}

void MZV::Graphics::Image::setBPP(int bpp)
{
	if (mBPP == bpp) {
		return;
	}

	try {
		switch (bpp) {
			case 32:
				mDIB = FreeImage_ConvertTo32Bits(mDIB);
			break;

			case 24:
				mDIB = FreeImage_ConvertTo24Bits(mDIB);
			break;

			case 16:
				mDIB = FreeImage_ConvertTo16Bits565(mDIB);
			break;

			case 8:
				mDIB = FreeImage_ConvertTo8Bits(mDIB);
			break;

			case 4:
				mDIB = FreeImage_ConvertTo4Bits(mDIB);
			break;

			default:
				throw Error::Exception(L"Geçersiz bir piksel başına bit değeri girdiniz", L"Resim Düzenleme Hatası");
			break;
		}

		if (mDIB) {
			mBPP = bpp;
		} else {
			throw Error::Exception(L"Resim piksel başına bit değerini değiştirirken bir hata oluştu", L"Resim Düzenleme Hatası");
		}
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Resim Düzenleme Hatası");
	} catch(Error::Exception& exception) {
		Error::ShowError(exception.getErrorMessage(),exception.getErrorTitle());
	} catch(std::exception& exception) {
		Error::ShowError(exception.what(), L"Resim Düzenleme Hatası");
	}
}

const RGBQUAD* MZV::Graphics::Image::getPalette(void) const
{
	return mPalette;
}

void MZV::Graphics::Image::setPalette(const RGBQUAD* palette)
{
	try {
		if (mBPP && (mBPP == 8 || mBPP == 4)) {
			if (mPalette) {
				delete[] mPalette;
				mPalette = nullptr;
			}

			mPalette = new RGBQUAD[256];
			std::copy(palette, palette + 256, mPalette);
		} else {
			throw Error::Exception(L"Resim paleti için bit başına düşen piksel değeri uyumlu değil", L"Resim Düzenleme Hatası");
		}
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Resim Düzenleme Hatası");
	} catch(Error::Exception& exception) {
		Error::ShowError(exception.getErrorMessage(),exception.getErrorTitle());
	} catch(std::exception& exception) {
		Error::ShowError(exception.what(), L"Resim Düzenleme Hatası");
	}
}

const FIBITMAP* MZV::Graphics::Image::getDIB(void) const
{
	return mDIB;
}

void MZV::Graphics::Image::setDIB(FIBITMAP* dib)
{
	if (mDIB == dib) {
		return;
	}

	try {
		if (dib) {
			if (mDIB) {
				FreeImage_Unload(mDIB);
				mDIB = nullptr;
			}

			mDIB = FreeImage_Clone(dib);

			if (mDIB == nullptr) {
			} else {
				throw Error::Exception(L"Resim kopyalanamadı", L"Resim Oluşturma hatası");
			}
		} else {
			throw Error::Exception(L"Resim bilgisi boş", L"Resim Oluşturma hatası");
		}
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Resim Oluşturma Hatası");

		reset();
	} catch(Error::Exception& exception) {
		Error::ShowError(exception.getErrorMessage(),exception.getErrorTitle());

		reset();
	} catch(std::exception& exception) {
		Error::ShowError(exception.what(), L"Resim Oluşturma Hatası");

		reset();
	}
}

bool MZV::Graphics::Image::load(void)
{
try {
		if (!mFileName.empty()) {
			if(!DoesPathOrFileExist(mFileName)) {
				throw Error::Exception(L"Resim dosyası bulunamadı!", L"Resim Yükleme Hatası");
			}

			mFormat = FreeImage_GetFIFFromFilenameU(mFileName.c_str());

			if ((mDIB = FreeImage_LoadU(mFormat, mFileName.c_str()))) {
				FreeImage_FlipVertical(mDIB);

				FIBITMAP* clone = FreeImage_ConvertTo32Bits(mDIB);

				if (clone) {
					if ((mWidth = FreeImage_GetWidth(mDIB))) {
						if ((mHeight = FreeImage_GetHeight(mDIB))) {
							if ((mBPP = FreeImage_GetBPP(mDIB))) {
								BYTE* bits = nullptr;
								if ((bits = FreeImage_GetBits(clone))) {
									if (setBits(bits, mWidth, mHeight)) {
										FreeImage_Unload(clone);

										return true;
									} else {
										reset();

										return false;
									}
								} else {
									throw Error::Exception(L"Resim dosyası pikselleri alınamadı :\n\n" + Error::GetLastErrorMessage(),
														   L"Resim Yükleme Hatası");
								}
							} else {
								throw Error::Exception(L"Resim dosyası pixel bilgisi okunamadı", L"Resim Yükleme Hatası");
							}
						} else {
							throw Error::Exception(L"Resim dosyası yüksekliği alınamadı", L"Resim Yükleme Hatası");
						}
					} else {
						throw Error::Exception(L"Resim dosyası genişliği alınamadı", L"Resim Yükleme Hatası");
					}
				} else {
					throw Error::Exception(L"Resim dosyası dönüştürülemedi!", L"Resim Yükleme Hatası");
				}
			} else {
				throw Error::Exception(L"Resim dosyası yüklenemedi!", L"Resim Yükleme Hatası");
			}
		} else {
			throw Error::Exception(L"Resim dosyası ismi seçilmemiş", L"Resim Yükleme Hatası");
		}
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Resim Yükleme Hatası");

		reset();

		return false;
	} catch(Error::Exception& exception) {
		Error::ShowError(exception.getErrorMessage(),exception.getErrorTitle());

		reset();

		return false;
	} catch(std::exception& exception) {
		Error::ShowError(exception.what(), L"Resim Yükleme Hatası");

		reset();

		return false;
	}
}

void MZV::Graphics::Image::reset(void)
{
	GraphicsElement::reset();

	mFileName.clear();

	mFormat = FIF_UNKNOWN;
	mBPP    = 0;

	if (mPalette) {
		delete[] mPalette;
		mPalette = nullptr;
	}

	if (mDIB) {
		FreeImage_Unload(mDIB);
	    mDIB = nullptr;
	}
}


void MZV::Graphics::Initialize(void)
{
	FreeImage_Initialise();
}

void MZV::Graphics::Deinitialize(void)
{
	FreeImage_DeInitialise();
}

BYTE* MZV::Graphics::SetBits(const BYTE* bits, const int width, const int height)
{
	BYTE* newBits{ nullptr };

	try {
		if (bits == nullptr) {
			throw Error::Exception(L"Bitler oluşturulmamış", L"Bit Oluşturma Hatası");
		}

		if (width == 0 || height == 0) {
			throw Error::Exception(L"Bitlerin boyutu seçilmemiş", L"Bit Oluşturma Hatası");
		}

		newBits = new BYTE[width * height * 4]{ 0 };
		std::copy(bits, bits + width * height * 4, newBits);

		return newBits;
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(std::string(ex.what()), L"Bit Oluşturma Hatası");

		return newBits;
	}  catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Oluşturma Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::SetBitsWidth(const BYTE* bits, const int width, const int height, int newWidth, int newHeight, const Color fillColor, bool scale)
{
	BYTE* newBits{ nullptr };

	try {
		if (scale) {
			newBits = ScaleBits(bits, width, height, newWidth, newHeight);

			return newBits;
		} else {
			newBits = new BYTE[newWidth * newHeight * 4];
			for (int i = 0; i < height; ++i) {
				for (int j = 0; j < newWidth; ++j) {
					int indexNew = (i * newWidth * 4) + (j * 4);
					int indexOld = (i * width * 4) + (j * 4);

					if (j > (width - 1)) {
						newBits[indexNew + 0] = fillColor.mRed;
						newBits[indexNew + 1] = fillColor.mGreen;
						newBits[indexNew + 2] = fillColor.mBlue;
						newBits[indexNew + 3] = fillColor.mAlpha;
					} else {
						newBits[indexNew + 0] = bits[indexOld + 0];
						newBits[indexNew + 1] = bits[indexOld + 1];
						newBits[indexNew + 2] = bits[indexOld + 2];
						newBits[indexNew + 3] = bits[indexOld + 3];
					}
				}
			}

			return newBits;
		}
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Bit Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Düzenleme Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::SetBitsHeight(const BYTE* bits, const int width, const int height, int newWidth, int newHeight, const Color fillColor, bool scale)
{
	BYTE* newBits{ nullptr };

	try {
		if (scale) {
			newBits = ScaleBits(bits, width, height, newWidth, newHeight);

			return newBits;
		} else {
			BYTE* newBits = new BYTE[newWidth * newHeight * 4];
			for (int i = 0; i < newHeight; ++i) {
				for (int j = 0; j < width; ++j) {
					int indexNew = (i * width * 4) + (j * 4);
					int indexOld = (i * width * 4) + (j * 4);

					if (i > (height - 1)) {
						newBits[indexNew + 0] = fillColor.mRed;
						newBits[indexNew + 1] = fillColor.mGreen;
						newBits[indexNew + 2] = fillColor.mBlue;
						newBits[indexNew + 3] = fillColor.mAlpha;
					} else {
						newBits[indexNew + 0] = bits[indexOld + 0];
						newBits[indexNew + 1] = bits[indexOld + 1];
						newBits[indexNew + 2] = bits[indexOld + 2];
						newBits[indexNew + 3] = bits[indexOld + 3];
					}
				}
			}

			return newBits;
		}
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Raster Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Raster Düzenleme Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::FlipBitsVertical(BYTE* bits, int width, int height)
{
	BYTE* newBits{ nullptr };

	try {
		newBits = new BYTE[width * height * 4]{ 0 };

		std::copy(bits, bits + width * height * 4, newBits);

		int index1 = 0;
		int index2 = 0;
		for (int i = 0; i < height / 2; ++i) {
			for (int j = 0; j < width; ++j) {
				index1 = i * width * 4 + j * 4;
				index2 = (height - i - 1) * width * 4 + j * 4;

				std::swap(*(newBits + index1 + 0), *(newBits + index2 + 0));
				std::swap(*(newBits + index1 + 1), *(newBits + index2 + 1));
				std::swap(*(newBits + index1 + 2), *(newBits + index2 + 2));
				std::swap(*(newBits + index1 + 3), *(newBits + index2 + 3));
			}
		}

		return newBits;
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Bit Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Düzenleme Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::FlipBitsHorizontal(BYTE* bits, int width, int height)
{
	BYTE* newBits{ nullptr };

	try {
		newBits = new BYTE[width * height * 4]{ 0 };

		std::copy(bits, bits + width * height * 4, newBits);

		int index1 = 0;
		int index2 = 0;
		for (int i = 0; i < height; ++i) {
			for (int j = 0; j < width / 2; ++j) {
				index1 = (i * width * 4 + j * 4);
				index2 = (i * width * 4 + (width - j - 1) * 4);

				std::swap(*(newBits + index1 + 0), *(newBits + index2 + 0));
				std::swap(*(newBits + index1 + 1), *(newBits + index2 + 1));
				std::swap(*(newBits + index1 + 2), *(newBits + index2 + 2));
				std::swap(*(newBits + index1 + 3), *(newBits + index2 + 3));
			}
		}

		return newBits;
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Bit Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Düzenleme Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::RotateBits(const BYTE* bits, const int width, const int height, float angle, int* newWidth, int* newHeight)
{
	BYTE* newBits{ nullptr };

	try {
		int xOrigin{ width / 2 };
		int yOrigin{ height / 2 };
		std::array<Math::Vector2D, 4> boundingBoxVertices;
		Math::Matrix3x3               rotationMatrix;

		boundingBoxVertices[0].setX((float)xOrigin * -1.0f);
		boundingBoxVertices[0].setY((float)yOrigin);
		boundingBoxVertices[1].setX((float)(width - xOrigin) * 1.0f);
		boundingBoxVertices[1].setY((float)yOrigin);
		boundingBoxVertices[2].setX((float)(width - xOrigin) * 1.0f);
		boundingBoxVertices[2].setY((float)(height - yOrigin) * -1.0f);
		boundingBoxVertices[3].setX((float)xOrigin * -1.0f);
		boundingBoxVertices[3].setY((float)(height - yOrigin) * -1.0f);

		int x{ 0 }, y{ 0 }, maxX{ 0 }, minX{ 0 }, maxY{ 0 }, minY{ 0 };
		rotationMatrix.setToRotation(angle);
		for (int i = 0; i < 4; ++i) {
			boundingBoxVertices[i] *= rotationMatrix;
			boundingBoxVertices[i].round();

			x = (int)boundingBoxVertices[i].getX();
			y = (int)boundingBoxVertices[i].getY();

			if (x < minX) {
				minX = x;
			}

			if (x > maxX) {
				maxX = x;
			}

			if (y < minY) {
				minY = y;
			}

			if (y > maxY) {
				maxY = y;
			}
		}

		*newWidth   = (maxX - minX);
		*newHeight  = (maxY - minY);
		newBits     = new BYTE[*newWidth * *newHeight * 4]{ 0 };
		int newOrgX = *newWidth / 2;
		int newOrgY = *newHeight / 2;
		Math::Vector2D pixVec{ 0.0f, 0.0f };
		int oldCoordX{ 0 };
		int oldCoordY{ 0 };
		int newCoordX{ 0 };
		int newCoordY{ 0 };
		int oldIndex{ 0 };
		int newIndex{ 0 };
		for (int i = 0; i < width * height; ++i) {
			oldCoordX = i % width - xOrigin;
			oldCoordY = yOrigin - i / width;

			pixVec.setX((float)oldCoordX);
			pixVec.setY((float)oldCoordY);

			pixVec *= rotationMatrix;
			pixVec.round();

			newCoordX = (int)pixVec.getX() + newOrgX;
			newCoordY = newOrgY - (int)pixVec.getY();

			oldIndex = i * 4;
			newIndex = (newCoordY * *newWidth * 4) + (newCoordX * 4);

			if (newIndex >= 0 && newIndex <= *newWidth * *newHeight * 4 - 4) {
				newBits[newIndex + 0] = bits[oldIndex + 0];
				newBits[newIndex + 1] = bits[oldIndex + 1];
				newBits[newIndex + 2] = bits[oldIndex + 2];
				newBits[newIndex + 3] = bits[oldIndex + 3];
			}
		}

		if (((int)angle) % 90) {
			int index{ 0 };
			int prevIndex{ 0 };
			int nextIndex{ 0 };
			for (int i = 0; i < *newHeight; ++i) {
				for (int j = 0; j < *newWidth; ++j) {
					if (j != 0 && j != *newWidth - 1) {
						index = (i * *newWidth * 4) + (j * 4);
						if (newBits[index + 0] == 0 &&
							newBits[index + 1] == 0 &&
							newBits[index + 2] == 0 &&
							newBits[index + 3] == 0) {
							prevIndex = (i * *newWidth * 4) + ((j - 1) * 4);
							nextIndex = (i * *newWidth * 4) + ((j + 1) * 4);
							newBits[index + 0] = (newBits[prevIndex + 0] + newBits[nextIndex + 0]) / 2;
							newBits[index + 1] = (newBits[prevIndex + 1] + newBits[nextIndex + 1]) / 2;
							newBits[index + 2] = (newBits[prevIndex + 2] + newBits[nextIndex + 2]) / 2;
							newBits[index + 3] = (newBits[prevIndex + 3] + newBits[nextIndex + 3]) / 2;

						}
					}

				}
			}
		}

		return newBits;
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Bit Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Düzenleme Hatası");

		return newBits;
	}
}

BYTE* MZV::Graphics::ScaleBits(const BYTE* bits, const int width, const int height, int newWidth, int newHeight)
{
	BYTE* newBits{ nullptr };

	try {
		float xFactor = (float)newWidth / (float)width;
		float yFactor = (float)newHeight / (float)height;

		newBits = new BYTE[newWidth * newHeight * 4];

		for (int C = 0; C < newWidth * newHeight * 4; C++) {
			newBits[C] = 1;
		}

		Math::Matrix3x3 scaleMatrix;
		Math::Vector2D  pixVector;

		scaleMatrix.setToScale(xFactor, yFactor);

		Math::Vector2D pixVec{ 0.0f, 0.0f };
		int oldCoordX{ 0 };
		int oldCoordY{ 0 };
		int newCoordX{ 0 };
		int newCoordY{ 0 };
		int oldIndex{ 0 };
		int newIndex{ 0 };
		for (int i = 0; i < width * height; ++i) {
			oldCoordX = i % width;
			oldCoordY = i / width;

			pixVec.setX((float)oldCoordX);
			pixVec.setY((float)oldCoordY);

			pixVec *= scaleMatrix;
			pixVec.round();

			newCoordX = (int)(pixVec.getX());
			newCoordY = (int)(pixVec.getY());

			oldIndex = i * 4;
			newIndex = (newCoordY * newWidth * 4) + ((newCoordX) * 4);

			if (newIndex >= 0 && newIndex <= ((newWidth * newHeight * 4) - 4)) {
				newBits[newIndex + 0] = bits[oldIndex + 0];
				newBits[newIndex + 1] = bits[oldIndex + 1];
				newBits[newIndex + 2] = bits[oldIndex + 2];
				newBits[newIndex + 3] = bits[oldIndex + 3];
			}
		}

		if (newWidth > width) {
			int index{ 0 };
			int prevIndex{ 0 };
			for (int i = 0; i < newHeight; ++i) {
				for (int j = 1; j < newWidth; ++j) {
					index = (i * newWidth * 4) + (j * 4);
					prevIndex = index - 4;
					if (newBits[index + 0] == 1 &&
						newBits[index + 1] == 1 &&
						newBits[index + 2] == 1 &&
						newBits[index + 3] == 1) {
						newBits[index + 0] = newBits[prevIndex + 0];
						newBits[index + 1] = newBits[prevIndex + 1];
						newBits[index + 2] = newBits[prevIndex + 2];
						newBits[index + 3] = newBits[prevIndex + 3];
					}
				}
			}
		}

		if (newHeight > height) {
			int index{ 0 };
			int upIndex{ 0 };
			for (int i = 0; i < newHeight; ++i) {
				for (int j = 0; j < newWidth; ++j) {
					index = (i * newWidth * 4) + (j * 4);
					upIndex = index - (newWidth * 4);
					if (newBits[index + 0] == 1 &&
						newBits[index + 1] == 1 &&
						newBits[index + 2] == 1 &&
						newBits[index + 3] == 1) {
						newBits[index + 0] = newBits[upIndex + 0];
						newBits[index + 1] = newBits[upIndex + 1];
						newBits[index + 2] = newBits[upIndex + 2];
						newBits[index + 3] = newBits[upIndex + 3];
					}
				}
			}
		}

		return newBits;
	} catch (Error::Exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return newBits;
	} catch (std::exception& ex) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Bit Düzenleme Hatası");

		return newBits;
	} catch(Exception& exception) {
		if (newBits) {
			delete[] newBits;
			newBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Bit Düzenleme Hatası");

		return newBits;
	}
}

MZV::CanerKurt::Map::Map()
 : mMapImage(nullptr), mProcessedMapImage(nullptr), mMapViewPort(nullptr),
   mWidth(0), mHeight(0),
   mPoints(nullptr),
   mAreaCount(0)
{}

MZV::CanerKurt::Map::Map(MZV::Graphics::Image* mapImage, TImage* viewPort)
 : mMapImage(mapImage), mProcessedMapImage(nullptr), mMapViewPort(viewPort),
   mWidth(mapImage->getWidth()), mHeight(mapImage->getHeight()),
   mPoints(new Point[mWidth * mHeight]{0}), mAreaCount(0)
{
	for(int i = 0;i < mHeight;++i) {
		for(int j = 0;j < mWidth;++j) {
			int index = i * mWidth + j;

			mPoints[index].mCoords.mX = j;
			mPoints[index].mCoords.mY = i;

			MZV::Graphics::Color color;
			mapImage->getPixelColor(j, i, color);

			mPoints[index].mColor.mAlpha = color.mAlpha;
			mPoints[index].mColor.mRed   = color.mRed;
			mPoints[index].mColor.mGreen = color.mGreen;
			mPoints[index].mColor.mBlue  = color.mBlue;

			if(color.mRed   == BORDER_COLOR.mRed   &&
			   color.mGreen == BORDER_COLOR.mGreen &&
			   color.mBlue  == BORDER_COLOR.mBlue) { //black pixel(border found!)
				mPoints[index].mIsBorder = true;
			} else { //not border!
				mPoints[index].mIsBorder = false;
			}

            mPoints[index].mIsProcessed = false;
		}
	}
}

MZV::CanerKurt::Map::Map(const Map& map)
 : mMapImage(map.mMapImage), mProcessedMapImage(nullptr), mMapViewPort(map.mMapViewPort),
   mWidth(map.mWidth), mHeight(map.mHeight),
   mPoints(new Point[mWidth * mHeight]), mAreaCount(map.mAreaCount)
{
	mPoints   = new Point[mWidth * mHeight]{0};
	std::copy(mPoints, mPoints + mWidth * mHeight, map.mPoints);
}

MZV::CanerKurt::Map::~Map()
{
	mMapImage = nullptr;

	if(mProcessedMapImage) {
		delete mProcessedMapImage;
	}

	mMapViewPort = nullptr;

	if(mPoints) {
		delete[] mPoints;
		mPoints = nullptr;
	}
}

const MZV::CanerKurt::Map& MZV::CanerKurt::Map::operator=(const Map& map)
{
	mMapImage          = map.mMapImage;

	mProcessedMapImage = map.mProcessedMapImage;

	mMapViewPort       = map.mMapViewPort;

	mWidth             = map.mWidth;

	mHeight            = map.mHeight;

	mPoints            = new Point[mWidth * mHeight]{0};
	std::copy(mPoints, mPoints + mWidth * mHeight, map.mPoints);

	mAreaCount         = map.mAreaCount;

	return *this;
}

void MZV::CanerKurt::Map::SetSize(const int width, const int height)
{
	Point* newPoints{ nullptr };

	try {
		//check if the arguments are valid
		if(width <= 0 || height <= 0) {
			throw Error::Exception(L"Harita boyutlandırma değerleri geçerli değil!",L"Harita Düzenleme Hatası");
		}

		//create new points
		newPoints = new Point[width * height]{0};

		//set new points coords and border data
		for(int i = 0;i < height;++i) {
			for(int j = 0;j < width;++j) {
				newPoints[i * height + j].mCoords.mX        = j;
				newPoints[i * height + j].mCoords.mY        = i;
				newPoints[i * height + j].mIsBorder = false;
			}
		}

		//copy old points border data to new points
		for (int i = 0; i < mHeight; ++i) {
			for (int j = 0; j < mWidth; ++j) {
				if(i < height && j < width) {
					int indexNew = i * width  + j;
					int indexOld = i * mWidth + j;

					newPoints[indexNew].mIsBorder = mPoints[indexOld].mIsBorder;
				}
			}
		}

		//delete old points, recreate and copy new points data
		if(mPoints) {
			delete[] mPoints;
			mPoints = nullptr;
		}

		mPoints = new Point[width * height];
		std::copy(mPoints, mPoints + width * height, newPoints);

		//all is good, set new dimensions
		mWidth  = width;
		mHeight = height;

		//delete temporary new points from the heap
		delete newPoints;
	} catch (Error::Exception& ex) {
		if (newPoints) {
			delete[] newPoints;
			newPoints = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());
	} catch (std::exception& ex) {
		if (newPoints) {
			delete[] newPoints;
			newPoints = nullptr;
		}

		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");
	} catch(Exception& exception) {
		if (newPoints) {
			delete[] newPoints;
			newPoints = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");
	}
}

void MZV::CanerKurt::Map::GetSize(int &width, int &height)
{
	width  = mWidth;
	height = mHeight;
}

void MZV::CanerKurt::Map::SetBorder(const int x, const int y)
{
	try {
		if(!IsOnTheMap(x, y)) {
			throw Error::Exception(L"Harita koordinatlari doğru değil!",L"Harita Düzenleme Hatası");
		} else {
			mPoints[y * mWidth + x].mIsBorder = true;
		}
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");
	}
}

void MZV::CanerKurt::Map::ClearBorder(const int x, const int y)
{
	try {
		if(!IsOnTheMap(x, y)) {
			throw Error::Exception(L"Harita koordinatlari doğru değil!",L"Harita Düzenleme Hatası");
		} else {
			mPoints[y * mWidth + x].mIsBorder = false;
		}

	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");
	}
}

bool MZV::CanerKurt::Map::IsBorder(const int x, const int y)
{
	try {
		if(!IsOnTheMap(x, y)) {
			return false;
		} else {
			return mPoints[y * mWidth + x].mIsBorder;
		}

	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Erişim Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::GetFirstUnprocessedPoint(int &x, int &y) const
{
	try {
		for(size_t i = 0;i < mWidth * mHeight;++i) {
			if(!mPoints[i].mIsBorder && !mPoints[i].mIsProcessed) {
			   x = mPoints[i].mCoords.mX;
			   y = mPoints[i].mCoords.mY;

			   return true;
			}
		}

		return false;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Erişim Hatası");

		return false;
	}
}

void MZV::CanerKurt::Map::Show()
{
	BYTE* processedMapImageBits{nullptr};

	try {
		//
		// Create the processed image bits
		//
		processedMapImageBits = new BYTE[mWidth * mHeight * 4]{0};

		for(int i = 0;i < mWidth * mHeight * 4;++i) {
			processedMapImageBits[i] = 255;
		}

		//
		// Copy the points to the processed image
		//
		for(size_t i = 0;i < mWidth*mHeight;++i) {
			int                  index = ((i / mWidth) * mWidth * 4) + ((i % mWidth) * 4);
			MZV::Graphics::Color pixel = mPoints[i].mColor;

			processedMapImageBits[index + 0] = pixel.mBlue;
			processedMapImageBits[index + 1] = pixel.mGreen;
			processedMapImageBits[index + 2] = pixel.mRed;
			processedMapImageBits[index + 3] = pixel.mAlpha;
		}

		//
		// Create processed map image
		//
		mProcessedMapImage = new MZV::Graphics::Image(L"İşlenmiş Harita Resmi", processedMapImageBits, mWidth, mHeight);

		delete processedMapImageBits;

		//
		// Scale the processed image to be contained in the viewport
		//

		int   processedMapImageWidth  = mProcessedMapImage->getWidth();
		int   processedMapImageHeight = mProcessedMapImage->getHeight();
		float widthScaleFactor        = MZV::Graphics::FRAME_HORIZONTAL_PIXEL_COUNT / (float)processedMapImageWidth;
		float heightScaleFactor       = MZV::Graphics::FRAME_VERTICAL_PIXEL_COUNT / (float)processedMapImageHeight;
		float chosenScaleFactor       = MZV::Math::Min(widthScaleFactor, heightScaleFactor);

		int scaledWidth  = chosenScaleFactor * processedMapImageWidth;
		int scaledHeight = chosenScaleFactor * processedMapImageHeight;

		if(!mProcessedMapImage->scale(scaledWidth, scaledHeight)) {
			throw Error::Exception(L"İşlenmiş harita resmi ölçeklendirilemedi!",L"Harita Görüntüleme Hatası");
		}

		//
		// Render the processed image to viewport
		//

		//determine start and end points to center the image horizontally or vertically
		int startX = (MZV::Graphics::FRAME_HORIZONTAL_PIXEL_COUNT - scaledWidth) / 2.0f;
		int endX   = startX + scaledWidth;
		int startY = (MZV::Graphics::FRAME_VERTICAL_PIXEL_COUNT - scaledHeight) / 2.0f;
		int endY   = startY + scaledHeight;

		//render
		for(int i = startY;i < endY;++i) {
			for(int j = startX;j < endX;++j) {
				MZV::Graphics::Color color;
				mProcessedMapImage->getPixelColor(j - startX, i - startY, color);

				mMapViewPort->Canvas->Pixels[j][i] = (TColor)RGB(color.mRed, color.mGreen, color.mBlue);
			}
		}
	} catch (Error::Exception& ex) {
		if(processedMapImageBits) {
			delete processedMapImageBits;
			processedMapImageBits = nullptr;
		}

		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());
	} catch (std::exception& ex) {
		if(processedMapImageBits) {
			delete processedMapImageBits;
			processedMapImageBits = nullptr;
		}

		Error::ShowError(ex.what(), L"Harita Görüntüleme Hatası");
	} catch(Exception& exception) {
		if(processedMapImageBits) {
			delete processedMapImageBits;
			processedMapImageBits = nullptr;
		}

		Error::ShowError(exception.Message.c_str(),L"Harita Görüntüleme Hatası");
	}
}

bool MZV::CanerKurt::Map::IsOnTheMap(const int x, const int y) const
{
	return x >= 0 && x < mWidth && y >= 0 && y < mHeight;
}

bool MZV::CanerKurt::Map::IsProcessed(const int x, const int y) const
{
	try {
		if(!IsOnTheMap(x, y)) {
			return false;
		} else {
			return mPoints[y * mWidth + x].mIsProcessed;
		}
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Erişim Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::SetProcessed(const int x, const int y)
{
	try {
		if(!IsOnTheMap(x, y)) {
			throw Error::Exception(L"Harita koordinatlari doğru değil!",L"Harita Düzenleme Hatası");
		} else {
			mPoints[y * mWidth + x].mIsProcessed = true;

			return true;
		}
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::SetColor(const int x, const int y, const Graphics::Color& color)
{
	try {
		if(!IsOnTheMap(x, y)) {
			throw Error::Exception(L"Harita koordinatlari doğru değil!",L"Harita Düzenleme Hatası");
		} else {
			mPoints[y * mWidth + x].mColor.mAlpha = color.mAlpha;
			mPoints[y * mWidth + x].mColor.mRed   = color.mRed;
			mPoints[y * mWidth + x].mColor.mGreen = color.mGreen;
			mPoints[y * mWidth + x].mColor.mBlue  = color.mBlue;

			return true;
		}
	}catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::IsThereBorderBetweenNeighborPoints(const int x1, const int y1, const int x2, const int y2)
{
	try {
		if(Math::Absolute(x1 - x2) == 0 || Math::Absolute(y2 - y1) == 0) { //check if they are on the same line
			return false;
		}

		// Get the intersection rectangle of the points' bounding boxes
		RECT boundingBoxOne = {x1 - 1, y1 - 1, x1 + 1, y1 + 1};
		RECT boundingBoxTwo = {x2 - 1, y2 - 1, x2 + 1, y2 + 1};
		RECT interRect{0, 0, 0, 0};

		//check if there is an intersection
		if(::IntersectRect(&interRect, &boundingBoxOne, &boundingBoxTwo)) {
			// extract the coords from the intersection rectangle
			std::array<Coordinates, 4> intersectCoords;

			intersectCoords[0] = {interRect.left, interRect.top};
			intersectCoords[1] = {interRect.right, interRect.top};
			intersectCoords[2] = {interRect.left, interRect.bottom};
			intersectCoords[3] = {interRect.right, interRect.bottom};

			// check if there is border on the both extracted points (diagonally)
			int borderCount{0};
			for(auto c : intersectCoords) {
				if((c.mX == x1 && c.mY == y1) || (c.mX == x2 && c.mY == y2)) { //is this an area point
					continue;
				} else {
					if(IsBorder(c.mX, c.mY)) { // is this a border point
						++borderCount;
					}
				}
			}

			if(borderCount == 2) {
                return true;
            }

			return false;
		}

		return false;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Erişim Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::FillPoint(const int fillX, const int fillY, const Graphics::Color fillColor)
{
	try {
		if(!IsOnTheMap(fillX, fillY)) {
			throw Error::Exception(L"Harita koordinatlari doğru değil!",L"Harita Erişim Hatası");
		}

		// create a manual stack for avoiding recursive function call stack overflow
		std::stack<Coordinates> points;

		points.push({fillX, fillY});
		while(!points.empty()) {
		   // pop first point from the stack
		   Coordinates c = points.top();
		   points.pop();

		   int x = c.mX;
		   int y = c.mY;

		   // check if point lies on the map and not a border and not processed before
		   if(IsOnTheMap(x, y) && !IsBorder(x, y) && !IsProcessed(x, y)) {
			   // set color of the point and mark as processed
			   SetColor(x, y, fillColor);
			   SetProcessed(x, y);

			   //add neighbor points to stack (if there is not a border between the current point)
			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x - 1), (y - 1))) {
				  points.push({(x - 1), (y - 1)});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x    ), (y - 1))) {
				  points.push({(x    ), (y - 1)});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x + 1), (y - 1))) {
				  points.push({(x + 1), (y - 1)});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x - 1), (y    ))) {
				  points.push({(x - 1), (y    )});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x + 1), (y    ))) {
				  points.push({(x + 1), (y    )});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x - 1), (y + 1))) {
				  points.push({(x - 1), (y + 1)});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x    ), (y + 1))) {
				  points.push({(x    ), (y + 1)});
			   }

			   if(!IsThereBorderBetweenNeighborPoints(x, y, (x + 1), (y + 1))) {
				  points.push({(x + 1), (y + 1)});
			   }
		   }
		}

		return true;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::GetAreaCount(int &count) const
{
	try {
	   count = mAreaCount;

	   return true;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Erişim Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Erişim Hatası");

		return false;
	}
}

bool MZV::CanerKurt::Map::SetAreaCount(const int count)
{
	try {
	   if(count < 0) {
		   throw Error::Exception(L"Harita alan sayısı geçerli değil!",L"Harita Düzenleme Hatası");
	   }

	   mAreaCount = count;

	   return true;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita Düzenleme Hatası");

		return false;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita Düzenleme Hatası");

		return false;
	}
}

MZV::CanerKurt::MapZoneCounter::MapZoneCounter(MZV::Bites::MapInterface* map)
 : mMapInterface(nullptr)
{
	Init(map);
}

MZV::CanerKurt::MapZoneCounter::~MapZoneCounter()
{
	mMapInterface = nullptr;
}

void MZV::CanerKurt::MapZoneCounter::Init(MZV::Bites::MapInterface* map)
{
	mMapInterface = map;
}

int MZV::CanerKurt::MapZoneCounter::Solve()
{
	try {
		//
		// Fill areas
		//
		std::srand(time(NULL));

		int x{0}, y{0};
		int zoneCount{0};
		while(mMapInterface->GetFirstUnprocessedPoint(x, y)) {
			Graphics::Color fillColor = {(BYTE)(rand() % 255), (BYTE)(rand() % 255), (BYTE)(rand() % 255), 255};

			mMapInterface->FillPoint(x, y, fillColor);
			++zoneCount;
		}

		mMapInterface->SetAreaCount(zoneCount);

		//
		// Return area count
		//
		int count{0};
		if(!mMapInterface->GetAreaCount(count)) {
			throw Error::Exception(L"Harita bölge sayısı alınamadı!", L"Harita İşleme Hatası");
		}

		return count;
	} catch (Error::Exception& ex) {
		Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return 0;
	} catch (std::exception& ex) {
		Error::ShowError(ex.what(), L"Harita İşleme Hatası");

		return 0;
	} catch(Exception& exception) {
		Error::ShowError(exception.Message.c_str(),L"Harita İşleme Hatası");

		return 0;
	}

}

//
// Global Functions
//

//---------------------------------------------------------------------------
// Show Error Function (Wide String Version)
//---------------------------------------------------------------------------
int MZV::Error::ShowError(const std::wstring message, const std::wstring caption)
{
	return MZV::ShowInfoBox(message, caption, IBT_ERROR);
}
//---------------------------------------------------------------------------
// Show Error Function (Basic String Version)
//---------------------------------------------------------------------------
int MZV::Error::ShowError(const std::string message, const std::wstring caption)
{
	return MZV::ShowInfoBox(message, caption, IBT_ERROR);
}
//---------------------------------------------------------------------------
// Show Error And Quit Function (Wide String Version)
//---------------------------------------------------------------------------
void MZV::Error::ShowErrorAndQuit(const std::wstring message, const std::wstring caption)
{
	Error::ShowError(message, caption);

	Application->Terminate();
}
//---------------------------------------------------------------------------
// Show Error And Quit Function (Basic String Version)
//---------------------------------------------------------------------------
void MZV::Error::ShowErrorAndQuit(const std::string message, const std::wstring caption)
{
	Error::ShowError(message, caption);

	Application->Terminate();
}
//---------------------------------------------------------------------------
// Get Last Error Message Function
//---------------------------------------------------------------------------
std::wstring MZV::Error::GetLastErrorMessage(void)
{
	//Get the error message, if any.
	DWORD errorMessageID = GetLastError();

	if (errorMessageID == 0) {
		return std::wstring(); //No error message has been recorded, return empty string
	}

	LPWSTR messageBuffer = nullptr;
	size_t size = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		                        NULL, errorMessageID, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPWSTR)&messageBuffer,
		                        0, NULL);

	std::wstring message(messageBuffer, size);

	//Free the buffer.
	LocalFree(messageBuffer);

	return message;
}

//---------------------------------------------------------------------------
// Get ApplicationDirectory Function
//---------------------------------------------------------------------------
std::wstring MZV::GetApplicationDirectory(void)
{
	WCHAR path[MAX_PATH];
	GetModuleFileNameW(NULL, path, MAX_PATH);

	std::wstring directory = path;

	directory = directory.substr(0, directory.find_last_of(L"\\"));

	return directory;
}

//---------------------------------------------------------------------------
// Convert String To Wide String Function
//---------------------------------------------------------------------------
std::wstring MZV::ConvertStringToWstring(const std::string& string)
{
	std::wstring temp(string.length(), L' ');
	std::copy(string.begin(), string.end(), temp.begin());

	return temp;
}

//---------------------------------------------------------------------------
// Convert Wide String To String Function
//---------------------------------------------------------------------------
std::string MZV::ConvertWstringToString(const std::wstring& wstring)
{
	std::setlocale(LC_ALL, "");

	const std::locale                                   locale("");
	typedef std::codecvt<wchar_t, char, std::mbstate_t> converter_type;
	const converter_type&                               converter = std::use_facet<converter_type>(locale);
	std::vector<char>                                   to(wstring.length() * converter.max_length());
	std::mbstate_t                                      state;
	const wchar_t*                                      from_next;
	char*                                               to_next;
	const converter_type::result                        result = converter.out(state, wstring.data(), wstring.data() + wstring.length(),
		                                                                       from_next, &to[0], &to[0] + to.size(), to_next);
	if (result == converter_type::ok or result == converter_type::noconv) {
		return std::string(&to[0], to_next);
	} else {
		return std::string{};
	}
}

//---------------------------------------------------------------------------
// Does Path or File Exist Function
//---------------------------------------------------------------------------
BOOL MZV::DoesPathOrFileExist(const std::wstring& dirOrFile)
{
	return PathFileExistsW(dirOrFile.c_str());
}

//---------------------------------------------------------------------------
// Show Info Box Function (Wide String Version)
//---------------------------------------------------------------------------
int MZV::ShowInfoBox(const std::wstring& message, const std::wstring& caption, MZV::INFO_BOX_TYPE messageType)
{
	UINT messageBoxType = 0;

	switch (messageType) {
		case MZV::IBT_EMPTY:
			messageBoxType = 0;
		break;

		case MZV::IBT_INFO:
			messageBoxType = MB_ICONINFORMATION;
		break;

		case MZV::IBT_WARNING:
			messageBoxType = MB_ICONWARNING;
		break;

		case MZV::IBT_ERROR:
			messageBoxType = MB_ICONERROR;
		break;

		default:
			messageBoxType = 0;
		break;
	}

	messageBoxType |= MB_SETFOREGROUND | MB_TOPMOST;

	return Application->MessageBox(message.c_str(), caption.c_str(), messageBoxType);
}
//---------------------------------------------------------------------------
// Show Info Box Function (Basic String Version)
//---------------------------------------------------------------------------
int MZV::ShowInfoBox(const std::string& message, const std::wstring& caption, MZV::INFO_BOX_TYPE messageType)
{
	UINT messageBoxType = 0;

	switch (messageType) {
		case MZV::IBT_EMPTY:
			messageBoxType = 0;
		break;

		case MZV::IBT_INFO:
			messageBoxType = MB_ICONINFORMATION;
		break;

		case MZV::IBT_WARNING:
			messageBoxType = MB_ICONWARNING;
		break;

		case MZV::IBT_ERROR:
			messageBoxType = MB_ICONERROR;
		break;

		default:
			messageBoxType = 0;
		break;
	}

	messageBoxType |= MB_SETFOREGROUND | MB_TOPMOST;

	return Application->MessageBox(MZV::ConvertStringToWstring(message).c_str(), caption.c_str(), messageBoxType);
}

//---------------------------------------------------------------------------
// Show Question Box Function
//---------------------------------------------------------------------------
int MZV::ShowQuestionBox(const std::wstring& message, const std::wstring& caption)
{
	return Application->MessageBox(message.c_str(),
								   caption.c_str(),
								   MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
}
