﻿/*
**
**                    MAP ZONE VIEWER - Version 1.0
**
** Map Zone Viewer loads an image file from hardrive, counts and displays
** the zones of the image divided by borderlines
**
** FreeImage library was used for the program to load images with different
** formats.
**
** Entirely designed and programmed by Caner'Trooper'Kurt on March 2021
** in Ankara, Turkey
**
** https://trooperthedreamer.com (all rights are reserved [(-:])
**
*/

#ifndef MAP_ZONE_VIEWER_HPP
#define MAP_ZONE_VIEWER_HPP

#include <vcl.h>
#include <tchar.h>
#include <FreeImage.h>
#include <vector>
#include <cmath>

//
// Enums
//

namespace MZV
{
    enum INFO_BOX_TYPE
	{
		IBT_EMPTY,
		IBT_INFO,
		IBT_WARNING,
		IBT_ERROR,
	};
}

//
// Classes
//

namespace MZV
{
	namespace Error
	{
		class Exception
		{
			public:

				Exception() = delete;
				Exception(const Exception& exception) = delete;
				Exception(std::wstring message, std::wstring title);

				Exception(const Exception&& exception) = delete;

				const Exception& operator=(const Exception& exception) = delete;

				std::wstring getErrorMessage(void) const;
				std::wstring getErrorTitle(void) const;

			private:

				std::wstring mErrorMessage;
				std::wstring mErrorTitle;
		};

		int          ShowError(const std::wstring message, const std::wstring caption);
		int          ShowError(const std::string message, const std::wstring caption);
		void         ShowErrorAndQuit(const std::wstring message, const std::wstring caption);
        void         ShowErrorAndQuit(const std::string message, const std::wstring caption);
		std::wstring GetLastErrorMessage(void);
	}

	namespace Math
	{
		class Matrix3x3;

		class Vector2D
		{
			public:

				Vector2D();
				Vector2D(float x, float y);
				Vector2D(const Vector2D& vector);


				Vector2D& operator=(const Vector2D& vector);
				float     operator[](unsigned int element) const;
				float&    operator[](unsigned int element);
				bool      operator==(const Vector2D& vector) const;
				bool      operator!=(const Vector2D& vector) const;
				Vector2D  operator*(float scalar) const;
				Vector2D  operator/(float scalar) const;
				Vector2D& operator*=(float scalar);
				Vector2D& operator/=(float scalar);
				Vector2D  operator*(const Matrix3x3& matrix) const;
				Vector2D& operator*=(const Matrix3x3& matrix);
				Vector2D  operator+(const Vector2D& vector) const;
				Vector2D  operator-(const Vector2D& vector) const;
				Vector2D& operator+=(const Vector2D& vector);
				Vector2D& operator-=(const Vector2D& vector);


				float getX(void) const;
				void  setX(float x);

				float getY(void) const;
				void  setY(float y);

				bool      isZero(void) const;
				Vector2D& setToZero(void);

				Vector2D  getNegate(void) const;
				Vector2D& setToNegate(void);

				float getMagnitude(void) const;

				Vector2D& normalize(void);

				Vector2D& round(void);

				Vector2D& clean(void);

			private:

				float mX;
				float mY;

				friend class Matrix3x3;
		};

		class Matrix3x3
		{
			public:

				Matrix3x3();
				Matrix3x3(float first, float second, float third,
						  float fourth, float fifth, float sixth,
						  float seventh, float eighth, float ninth);
				Matrix3x3(const Matrix3x3& matrix);


				Matrix3x3& operator=(const Matrix3x3& matrix);
				float      operator()(unsigned int row, unsigned int col) const;
				float&     operator()(unsigned int row, unsigned int col);
				Matrix3x3  operator*(float scalar) const;
				Matrix3x3& operator*=(float scalar);
				Matrix3x3  operator/(float scalar) const;
				Matrix3x3& operator/=(float scalar);
				Matrix3x3  operator*(const Matrix3x3& matrix) const;
				Matrix3x3& operator*=(const Matrix3x3& matrix);
				Vector2D   operator*(const Vector2D& vector) const;


				Matrix3x3  getTranspose(void) const;
				Matrix3x3& setToTranspose(void);

				bool       isIdentity(void) const;
				Matrix3x3& setToIdentity(void);

				bool       isZero(void) const;
				Matrix3x3& setToZero(void);

				float getDeterminant(void) const;

				Matrix3x3& setToTranslation(float translateX, float translateY);
				Matrix3x3& setToRotation(float angle);
				Matrix3x3& setToScale(float scaleX, float scaleY);

				Matrix3x3& clean(void);

			private:

				float m11, m12, m13;
				float m21, m22, m23;
				float m31, m32, m33;

				friend class Vector2D;
		};

		constexpr float EPSILON = 1.0e-6f;
		constexpr float PI      = 3.1415926535897932384626433832795f;
		constexpr float HALF_PI = 1.5707963267948966192313216916398f;
		constexpr float TWO_PI  = 2.0f * PI;

		int       Absolute(int number);
		float     Absolute(float number);
		int       Min(int number1, int number2);
		float     Min(float number1, float number2);
		int       Max(int number1, int number2);
		float     Max(float number1, float number2);
		bool      IsOdd(int number);
		bool      IsEven(int number);
		bool      IsZero(float number);
		bool      AreEqual(float number1, float number2);
		float     Round(float number);
		float     Ceil(float number);
		float     Floor(float number);
		float     Square(float number);
		int       Square(int number);
		float     Power(float base, float exponent);
		float     SquareRoot(float number);
		float     DegreeToRadian(float degree);
		float     RadianToDegree(float radian);
		float     Sine(float degree);
		float     Cosine(float degree);
		float     Tangent(float degree);
		float	  ArcSine(float degree);
		float	  ArcCosine(float degree);
		float     ArcTangent(float degree);
		float     Distance(Vector2D& vector1, Vector2D& vector2);
		float     AngleBetween(Vector2D& vector1, Vector2D& vector2);
		float     DotProduct(Vector2D& vector1, Vector2D& vector2);
		float     CrossProduct(Vector2D& vector1, Vector2D& vector2);
		bool      GetLineSegmentIntersection(POINT line1Start, POINT line1End, POINT line2Start, POINT line2End, Vector2D& intersection);
		Vector2D  operator*(float scalar, Vector2D& vector);
		Matrix3x3 operator*(float scalar, Matrix3x3& matrix);
	}

	namespace Graphics
	{
		constexpr int FRAME_HORIZONTAL_PIXEL_COUNT = 1280;
		constexpr int FRAME_VERTICAL_PIXEL_COUNT   = 720;

		struct Color
		{
			BYTE mRed{0};
			BYTE mGreen{0};
			BYTE mBlue{0};
			BYTE mAlpha{0};
		};

		class GraphicsElement
		{
			public:

				GraphicsElement();
				GraphicsElement(const std::wstring& name, const BYTE* bits = nullptr, const int width = 0, const int height = 0);
				GraphicsElement(const GraphicsElement& element);
				virtual ~GraphicsElement();


				virtual const GraphicsElement& operator=(const GraphicsElement& element);
				virtual bool                   operator==(const GraphicsElement& element) const; //compares by the bits
				virtual bool                   operator!=(const GraphicsElement& element) const; //compares by the bits


				virtual const std::wstring& getName(void) const;
				virtual void                setName(const std::wstring& name);

				virtual int  getWidth(void) const;
				virtual bool setWidth(const int width, const Color fillColor = Color{}, bool scale = false);

				virtual int  getHeight(void) const;
				virtual bool setHeight(const int height, const Color fillColor = Color{}, bool scale = false);

				virtual BYTE* getBits(void) const;
				virtual bool  setBits(const BYTE* bits, const int width, const int height);

				virtual bool getPixelColor(const int x, const int y, Color& color) const;
				virtual bool setPixelColor(const int x, const int y, Color color);

				/**
				  Rotates the element by the angle around the center

				  angle is in degrees
				*/
				virtual bool rotate(float angle);
				/**
				  Scales the element to new width and new height
				*/
				virtual bool scale(int width, int height);
				/**
				  Scales the element to input rectangle
				*/
				virtual bool scale(RECT r);
				/**
				  Flips the element vertically
				*/
				virtual bool flipVertical(void);
				/**
				  Flips the element horizontally
				*/
				virtual bool flipHorizontal(void);
				/**
				  Resets all members of the element
				*/
				virtual void reset(void);

			protected:

				std::wstring mName;
				int          mWidth;
				int          mHeight;
				BYTE*        mBits;
		};

		class Image : public GraphicsElement
		{
			public:

				Image();
				Image(const std::wstring& fileName, const std::wstring& name);
				Image(const std::wstring& name, const BYTE* bits, const int width, const int height);
				Image(const Image& image);
				Image(const Image& image, const std::wstring& name);
				virtual ~Image();


				const Image& operator=(const Image& image);


				const std::wstring& getFileName(void) const;
				void                setFileName(const std::wstring& fileName);

				const FREE_IMAGE_FORMAT getFormat(void) const;
				void                    setFormat(const FREE_IMAGE_FORMAT format);

				int  getBPP(void) const;
				void setBPP(const int bpp);

				const RGBQUAD* getPalette(void) const;
				void           setPalette(const RGBQUAD* palette);

				const FIBITMAP* getDIB(void) const;
				void            setDIB(FIBITMAP* dib);

				/**
				  Loads image file from the harddrive
				*/
				bool load(void);
				void reset(void);

			private:

				std::wstring      mFileName;
				FREE_IMAGE_FORMAT mFormat;
				int               mBPP;      //Bits per pixel for the original DIB
				RGBQUAD*          mPalette;
				FIBITMAP*         mDIB;      //Original DIB
		};

		void  Initialize(void);
		void  Deinitialize(void);
		BYTE* SetBits(const BYTE* bits, const int width, const int height);
		BYTE* SetBitsWidth(const BYTE* bits, const int width, const int height, int newWidth, int newHeight, const Color fillColor = Color{}, bool scale = false);
		BYTE* SetBitsHeight(const BYTE* bits, const int width, const int height, int newWidth, int newHeight, const Color fillColor = Color{}, bool scale = false);
		BYTE* FlipBitsVertical(BYTE* bits, int width, int height);
		BYTE* FlipBitsHorizontal(BYTE* bits, int width, int height);
		BYTE* RotateBits(const BYTE* bits, const int width, const int height, float angle, int* newWidth, int* newHeight);
		BYTE* ScaleBits(const BYTE* bits, const int width, const int height, int newWidth, int newHeight);
	}
}

namespace MZV
{
	namespace CanerKurt
	{
		struct Coordinates
		{
			int mX;
			int mY;
		};

		struct Point
		{
			Coordinates     mCoords;
			bool            mIsBorder;
			bool            mIsProcessed;
			Graphics::Color mColor;
		};

		constexpr Graphics::Color BORDER_COLOR{0, 0, 0, 255};
	}

	namespace Bites
	{
        class MapInterface
		{
			public:

				virtual ~MapInterface() {};
				// Creates a map of given size.
				virtual void SetSize(const int width, const int height) = 0;
				// Returns size of map to solve.
				virtual void GetSize(int &width, int &height) = 0;
				// Sets border at given point.
				virtual void SetBorder(const int x, const int y) = 0;
				// Clears border at given point.
				virtual void ClearBorder(const int x, const int y) = 0;
				// Checks if there is a border at given point.
				virtual bool IsBorder(const int x, const int y) = 0;
				// Show map contents.
				virtual void Show() = 0;

				// Checks if a given point is on the map
				virtual bool IsOnTheMap(const int x, const int y) const = 0;
				// Checks if a given point is processed
				virtual bool IsProcessed(const int x, const int y) const = 0;
				// Marks a given point as processed
				virtual bool SetProcessed(const int x, const int y) = 0;
				// Sets the color of a given point
				virtual bool SetColor(const int x, const int y, const Graphics::Color& color) = 0;
				// Checks if there is a border between two given neighbor points
				virtual bool IsThereBorderBetweenNeighborPoints(const int x1, const int y1, const int x2, const int y2) = 0;
				// Fills a point with an extended boundary fill algorithm
				virtual bool FillPoint(const int fillX, const int fillY, const Graphics::Color fillColor) = 0;
				// Gets the first unprocessed point on the map
				virtual bool GetFirstUnprocessedPoint(int &x, int &y) const = 0;
				// Gets the area count of the map
				virtual bool GetAreaCount(int &count) const = 0;
				// Sets area count of the map
				virtual bool SetAreaCount(const int count) = 0;
		};

		class ZoneCounterInterface
		{
			public:

				virtual ~ZoneCounterInterface() {}
				// Feeds map instance into solution class, and initialize.
				virtual void Init(MapInterface * map) = 0;
				// Counts zones in provided map, and return result.
				virtual int Solve() = 0;
		};
	}

	namespace CanerKurt
	{
		class Map : public MZV::Bites::MapInterface
		{
			public:

				Map();
				Map(MZV::Graphics::Image* mapImage, TImage* viewPort);
				Map(const Map& map);
				virtual ~Map();


				const Map& operator=(const Map& map);


				void SetSize(const int width, const int height);
				void GetSize(int &width, int &height);
				void SetBorder(const int x, const int y);
				void ClearBorder(const int x, const int y);
				bool IsBorder(const int x, const int y);
				void Show();

				bool IsOnTheMap(const int x, const int y)                                                       const;
				bool IsProcessed(const int x, const int y)                                                      const;
				bool SetProcessed(const int x, const int y);
				bool SetColor(const int x, const int y, const Graphics::Color& color);
				bool IsThereBorderBetweenNeighborPoints(const int x1, const int y1, const int x2, const int y2);
				bool FillPoint(const int fillX, const int fillY, const Graphics::Color fillColor);
				bool GetFirstUnprocessedPoint(int &x, int &y)                                                   const;
				bool GetAreaCount(int &count)                                                                   const;
				bool SetAreaCount(const int count);

			private:

				MZV::Graphics::Image* mMapImage;
				MZV::Graphics::Image* mProcessedMapImage;
				TImage*               mMapViewPort;
				int                   mWidth;
				int                   mHeight;
				Point*                mPoints;
				int                   mAreaCount;

		};

		class MapZoneCounter : public MZV::Bites::ZoneCounterInterface
		{
			public:

				MapZoneCounter() = delete;
				MapZoneCounter(::MZV::Bites::MapInterface* map);
				MapZoneCounter(const MapZoneCounter& mapZoneCounter) = delete;
				virtual ~MapZoneCounter();


				const MapZoneCounter& operator=(const MapZoneCounter& mapZoneCounter) = delete;


				void Init(MZV::Bites::MapInterface* map);
				int  Solve();

			private:

				MZV::Bites::MapInterface* mMapInterface;

        };
	}
}

//
// Global Functions
//

namespace MZV
{
	std::wstring GetApplicationDirectory(void);
	std::wstring ConvertStringToWstring(const std::string& string);
	std::string  ConvertWstringToString(const std::wstring& wstring);
	BOOL         DoesPathOrFileExist(const std::wstring& dirOrFile);
	int          ShowInfoBox(const std::wstring& message, const std::wstring& caption, INFO_BOX_TYPE messageType = IBT_EMPTY);
	int          ShowInfoBox(const std::string& message, const std::wstring& caption, INFO_BOX_TYPE messageType = IBT_EMPTY);
	int          ShowQuestionBox(const std::wstring& message, const std::wstring& caption);
}
#endif // MAP_ZONE_VIEWER_HPP
